@@ -22,7 +22,14 @@ switchGRP <- function(x, na.last = TRUE, decreasing = FALSE, starts = FALSE,
   .Call(C_radixsort, na.last, decreasing, starts, group.sizes, sort, z)
 }
 
-group <- function(x, starts = FALSE, group.sizes = FALSE) {
+group <- function(..., starts = FALSE, group.sizes = FALSE) {
+  x <- if(...length() == 1L) ..1 else list(...)
+  g <- .Call(C_group, x, starts, group.sizes)
+  oldClass(g) <- c("qG", "na.included")
+  g
+}
+
+groupv <- function(x, starts = FALSE, group.sizes = FALSE) {
   g <- .Call(C_group, x, starts, group.sizes)
   oldClass(g) <- c("qG", "na.included")
   g
@@ -106,14 +113,15 @@ GRP.default <- function(X, by = NULL, sort = .op[["sort"]], decreasing = FALSE,
       } else if(is.character(by)) {
         namby <- by
         by <- ckmatch(by, attr(X, "names"))
-      } else if(is.numeric(by)) {
-        by <- as.integer(by)
+      } else {
+        by <- if(is.numeric(by)) as.integer(by) else if(is.logical(by)) which(by) else if(is.function(by)) which(vapply(unattrib(X), by, TRUE)) else
+              stop("by needs to be either a one-sided formula, character column names, column indices, a logical vector or selector function!")
         namby <- attr(X, "names")[by]
         if(is.null(namby)) {
           namby <- paste0("Group.", seq_along(by))
           attr(X, "names") <- paste0("Group.", seq_along(unclass(X))) # best ?
         }
-      } else stop("by needs to be either a one-sided formula, character column names or column indices!")
+      }
       o <- switchGRP(.subset(X, by), na.last, decreasing, return.groups || !use.group, TRUE, sort, use.group)
     }
   } else {
@@ -131,10 +139,10 @@ GRP.default <- function(X, by = NULL, sort = .op[["sort"]], decreasing = FALSE,
   if(return.groups) {
       # if unit groups, don't subset rows...
       if(length(gs) == length(o) && (use.group || sorted)) {
-        ust <- NULL
+        ust <- st
         groups <- if(is.list(X)) .Call(C_subsetCols, X, by, FALSE) else `names<-`(list(X), namby)
       } else {
-        ust <- if(use.group || sorted) st else .Call(C_subsetVector, o, st, FALSE) # o[st]
+        ust <- if(use.group || sorted) st else if(length(gs) == length(o)) o else .Call(C_subsetVector, o, st, FALSE) # o[st]
         groups <- if(is.list(X)) .Call(C_subsetDT, X, ust, by, FALSE) else
           `names<-`(list(.Call(C_subsetVector, X, ust, FALSE)), namby) # subsetVector preserves attributes (such as "label")
       }
@@ -149,16 +157,16 @@ GRP.default <- function(X, by = NULL, sort = .op[["sort"]], decreasing = FALSE,
                         groups = groups,
                         group.vars = namby,
                         ordered = c(ordered = sort, sorted = sorted),
-                        order = if(return.order && !use.group) .Call(C_setAttributes, o, ao) else NULL, # `attributes<-`(o, attributes(o)[-2L]) This does a shallow copy on newer R versions # `attr<-`(o, "group.sizes", NULL): This deep-copies it..
+                        order = if(return.order && !use.group) `attributes<-`(o, ao) else NULL, # `attributes<-`(o, attributes(o)[-2L]) This does a shallow copy on newer R versions # `attr<-`(o, "group.sizes", NULL): This deep-copies it..
                         group.starts = ust, # Does not need to be computed by group()
                         call = if(call) match.call() else NULL), "GRP"))
 }
 
 is_GRP <- function(x) inherits(x, "GRP")
-is.GRP <- function(x) {
-  .Deprecated(msg = "'is.GRP' was renamed to 'is_GRP'. It will be removed end of 2023, see help('collapse-renamed').")
-  inherits(x, "GRP")
-}
+# is.GRP <- function(x) {
+#   .Deprecated(msg = "'is.GRP' was renamed to 'is_GRP'. It will be removed end of 2023, see help('collapse-renamed').")
+#   inherits(x, "GRP")
+# }
 
 length.GRP <- function(x) length(x[[2L]])
 
@@ -237,20 +245,25 @@ print.GRP <- function(x, n = 6, ...) {
   }
 }
 
-plot.GRP <- function(x, breaks = "auto", type = "s", horizontal = FALSE, ...) {
+plot.GRP <- function(x, breaks = "auto", type = "l", horizontal = FALSE, ...) {
   # if(!missing(...)) unused_arg_action(match.call(), ...)
-  oldpar <- par(mfrow = if(horizontal) 1:2 else 2:1, mar = c(3.9,4.1,2.1,1), mgp = c(2.5,1,0))
-  on.exit(par(oldpar))
+  if(x[[1L]] <= 1e4) {
+    oldpar <- par(mfrow = if(horizontal) 1:2 else 2:1,
+                  mar = c(3.9,4.1,2.1,1), mgp = c(2.5,1,0))
+    on.exit(par(oldpar))
+  }
   if(breaks == "auto") {
-    ugs <- length(funique(x[[3L]]))
+    ugs <- fndistinct.default(x[[3L]])
     breaks <- if(ugs > 80) 80 else ugs
   }
-  plot(seq_len(x[[1L]]), x[[3L]], type = type, xlab = "Group id", ylab = "Group Size",
-       main = paste0("Sizes of ", x[[1L]], if(isTRUE(any(x[[6L]]))) " Ordered"  else if(anyNA(x[[6L]])) "" else " Unordered", " Groups"), frame.plot = FALSE, ...)
+  if(x[[1L]] <= 1e4) plot(seq_len(x[[1L]]), x[[3L]], type = type, xlab = "Group id", ylab = "Group Size",
+                          xlim = c(1L, x[[1L]]), ylim = c(0L, bmax(x[[3L]])),
+                          main = paste0("Sizes of ", x[[1L]], if(isTRUE(any(x[[6L]]))) " Ordered" else if(anyNA(x[[6L]])) "" else " Unordered", " Groups"), frame.plot = FALSE, ...)
   # grid()
   if(breaks == 1L) plot(x[[3L]][1L], x[[1L]], type = "h", ylab = "Frequency", xlab = "Group Size",
                         main = "Histogram of Group Sizes", frame.plot = FALSE, ...) else
-  hist(x[[3L]], breaks, xlab = "Group Size", main = "Histogram of Group Sizes", ...)
+  hist(x[[3L]], breaks, xlab = "Group Size",
+       main = paste0("Histogram of Group Sizes", if(x[[1L]] > 1e4) paste0(" (N = ", x[[1L]], ")") else ""), ...)
 }
 
 as_factor_GRP <- function(x, ordered = FALSE, sep = ".") { # , ...
@@ -271,10 +284,10 @@ as_factor_GRP <- function(x, ordered = FALSE, sep = ".") { # , ...
   f
 }
 
-as.factor_GRP <- function(x, ordered = FALSE) {
-  .Deprecated(msg = "'as.factor_GRP' was renamed to 'as_factor_GRP'. It will be removed end of 2023, see help('collapse-renamed').")
-  as_factor_GRP(x, ordered)
-}
+# as.factor_GRP <- function(x, ordered = FALSE) {
+#   .Deprecated(msg = "'as.factor_GRP' was renamed to 'as_factor_GRP'. It will be removed end of 2023, see help('collapse-renamed').")
+#   as_factor_GRP(x, ordered)
+# }
 
 finteraction <- function(..., factor = TRUE, ordered = FALSE, sort = factor && .op[["sort"]], method = "auto", sep = ".") { # does it drop levels ? -> Yes !
   X <- if(...length() == 1L && is.list(..1)) ..1 else list(...)
@@ -415,6 +428,17 @@ fgroup_by <- function(.X, ..., sort = .op[["sort"]], decreasing = FALSE, na.last
 
 gby <- fgroup_by
 
+group_by_vars <- function(X, by = NULL, ...) {
+  clx <- oldClass(X)
+  oldClass(X) <- NULL
+  m <- match(c("GRP_df", "grouped_df", "data.frame"), clx, nomatch = 0L)
+  if(length(by)) by <- cols2int(by, X, names(X), FALSE)
+  attr(X, "groups") <- GRP.default(X[by], NULL, ..., call = FALSE) # Need to unclass because of sf and regrouping! (and some functions expect unclassed)
+  oldClass(X) <- c("GRP_df",  if(length(mp <- m[m != 0L])) clx[-mp] else clx, "grouped_df", if(m[3L]) "data.frame")
+  if(any(clx == "data.table")) return(alc(X))
+  X
+}
+
 print.GRP_df <- function(x, ...) {
   print(fungroup(x), ...) # better !! (the method could still print groups attribute etc. ) And can also get rid of .rows() in fgroup_by and other fuzz..
   # but better keep for now, other functions in dplyr might check this and only preserve attributes if they exist. -> Nah. select(UGA_sf, addr_cname) doesn't work anyway..
@@ -478,6 +502,7 @@ print.invisible <- function(x, ...) cat("")
 `[[.GRP_df` <-  function(x, ...) UseMethod("[[", fungroup(x)) # function(x, ..., exact = TRUE) .subset2(x, ..., exact = exact)
 `[<-.GRP_df` <- function(x, ..., value) UseMethod("[<-", fungroup(x))
 `[[<-.GRP_df` <- function(x, ..., value) UseMethod("[[<-", fungroup(x))
+`names<-.GRP_df` <- function(x, value) `oldClass<-`(`names<-`(unclass(x), value), oldClass(x))
 
 # Produce errors...
 # print_GRP_df_core <- function(x) {
@@ -568,10 +593,10 @@ GRP.grouped_df <- function(X, ..., return.groups = TRUE, call = TRUE) {
 }
 
 is_qG <- function(x) is.integer(x) && inherits(x, "qG")
-is.qG <- function(x) {
-  .Deprecated(msg = "'is.qG' was renamed to 'is_qG'. It will be removed end of 2023, see help('collapse-renamed').")
-  inherits(x, "qG")
-}
+# is.qG <- function(x) {
+#   .Deprecated(msg = "'is.qG' was renamed to 'is_qG'. It will be removed end of 2023, see help('collapse-renamed').")
+#   inherits(x, "qG")
+# }
 
 na_rm2 <- function(x, sort) {
   if(sort) return(if(is.na(x[length(x)])) x[-length(x)] else x)
@@ -670,10 +695,10 @@ as_factor_qG <- function(x, ordered = FALSE, na.exclude = TRUE) {
   return(`attributes<-`(x, list(levels = groups, class = clx)))
 }
 
-as.factor_qG <- function(x, ordered = FALSE, na.exclude = TRUE) {
-  .Deprecated(msg = "'as.factor_qG' was renamed to 'as_factor_qG'. It will be removed end of 2023, see help('collapse-renamed').")
-  as_factor_qG(x, ordered, na.exclude)
-}
+# as.factor_qG <- function(x, ordered = FALSE, na.exclude = TRUE) {
+#   .Deprecated(msg = "'as.factor_qG' was renamed to 'as_factor_qG'. It will be removed end of 2023, see help('collapse-renamed').")
+#   as_factor_qG(x, ordered, na.exclude)
+# }
 
 qF <- function(x, ordered = FALSE, na.exclude = TRUE, sort = .op[["sort"]], drop = FALSE,
                keep.attr = TRUE, method = "auto") {
@@ -715,6 +740,7 @@ qG <- function(x, ordered = FALSE, na.exclude = TRUE, sort = .op[["sort"]],
         ax <- if(return.groups) list(N.groups = attr(x, "N.groups"), groups = attr(x, "groups"), class = newclx) else
           list(N.groups = attr(x, "N.groups"), class = newclx)
       }
+      if(identical(ax, attributes(x))) return(x)
       return(`attributes<-`(x, ax))
     }
     newclx <- c(if(ordered) "ordered", "qG", "na.included")
@@ -854,13 +880,13 @@ fduplicated <- function(x, all = FALSE) {
   if(all) {
     g <- .Call(C_group, x, FALSE, FALSE)
     ng <- attr(g, "N.groups")
-    if(ng == length(g)) return(.Call(C_alloc, FALSE, length(g), TRUE))
+    if(ng == length(g)) return(logical(length(g)))
     gs <- .Call(C_fwtabulate, g, NULL, ng, FALSE)
     return(.Call(C_subsetVector, gs != 1L, g, FALSE))
   }
   g <- .Call(C_group, x, TRUE, FALSE)
   starts <- attr(g, "starts")
-  if(length(starts) == length(g)) return(.Call(C_alloc, FALSE, length(g), TRUE))
+  if(length(starts) == length(g)) return(logical(length(g)))
   .Call(C_setcopyv, .Call(C_alloc, TRUE, length(g), TRUE), starts, FALSE, FALSE, TRUE, TRUE)
 }
 
