@@ -116,7 +116,7 @@ pasteclass <- function(x) if(length(cx <- class(x)) > 1L) paste(cx, collapse = "
 
 vclasses <- function(X, use.names = TRUE) {
   if(is.atomic(X)) return(pasteclass(X))
-  vapply(X, pasteclass, character(1L), USE.NAMES = use.names) # unattrib(X): no names
+  vapply(X, pasteclass, "", USE.NAMES = use.names) # unattrib(X): no names
 }
 
 # https://github.com/wch/r-source/blob/4a409a1a244d842a3098d2783c5b63c9661fc6be/src/main/util.c
@@ -156,7 +156,7 @@ vtypes <- function(X, use.names = TRUE) {
   res <- R_types[.Call(C_vtypes, X, 0L)]
   if(use.names) names(res) <- attr(X, "names")
   res
-  # vapply(X, typeof, character(1L)) # unattrib(X): no names
+  # vapply(X, typeof, "") # unattrib(X): no names
 }
 
 vlengths <- function(X, use.names = TRUE) .Call(C_vlengths, X, use.names)
@@ -165,9 +165,9 @@ namlab <- function(X, class = FALSE, attrn = "label", N = FALSE, Ndistinct = FAL
   if(!is.list(X)) stop("namlab only works with lists")
   res <- list(Variable = attr(X, "names"))
   attributes(X) <- NULL
-  if(class) res$Class <- vapply(X, pasteclass, character(1), USE.NAMES = FALSE)
+  if(class) res$Class <- vapply(X, pasteclass, "", USE.NAMES = FALSE)
   if(N) res$N <- fnobs.data.frame(X)
-  if(Ndistinct) res$Ndist <- fndistinct.data.frame(X)
+  if(Ndistinct) res$Ndist <- fndistinct.data.frame(X, na.rm = TRUE)
   res$Label <- vlabels(X, attrn, FALSE)
   attr(res, "row.names") <- c(NA_integer_, -length(X))
   oldClass(res) <- "data.frame"
@@ -318,16 +318,16 @@ addAttributes <- function(x, a) .Call(C_setAttributes, x, c(attributes(x), a))
 
 
 is_categorical <- function(x) !is.numeric(x)
-is.categorical <- function(x) {
-  .Deprecated(msg = "'is.categorical' was renamed to 'is_categorical'. It will be removed end of 2023, see help('collapse-renamed').")
-  !is.numeric(x)
-}
+# is.categorical <- function(x) {
+#   .Deprecated(msg = "'is.categorical' was renamed to 'is_categorical'. It will be removed end of 2023, see help('collapse-renamed').")
+#   !is.numeric(x)
+# }
 
 is_date <- function(x) inherits(x, c("Date","POSIXlt","POSIXct"))
-is.Date <- function(x) {
-  .Deprecated(msg = "'is.Date' was renamed to 'is_date'. It will be removed end of 2023, see help('collapse-renamed').")
-  inherits(x, c("Date","POSIXlt","POSIXct"))
-}
+# is.Date <- function(x) {
+#   .Deprecated(msg = "'is.Date' was renamed to 'is_date'. It will be removed end of 2023, see help('collapse-renamed').")
+#   inherits(x, c("Date","POSIXlt","POSIXct"))
+# }
 
 # more consistent with base than na_rm
 # na.rm <- function(x) { # cpp version available, but not faster !
@@ -501,6 +501,16 @@ as_numeric_factor <- function(X, keep.attr = TRUE) {
   res
 }
 
+as_integer_factor <- function(X, keep.attr = TRUE) {
+  if(is.atomic(X)) if(keep.attr) return(ffka(X, as.integer)) else
+    return(as.integer(attr(X, "levels"))[X])
+  res <- duplAttributes(lapply(unattrib(X),
+    if(keep.attr) (function(y) if(is.factor(y)) ffka(y, as.integer) else y) else
+                  (function(y) if(is.factor(y)) as.integer(attr(y, "levels"))[y] else y)), X)
+  if(inherits(X, "data.table")) return(alc(res))
+  res
+}
+
 as_character_factor <- function(X, keep.attr = TRUE) {
   if(is.atomic(X)) if(keep.attr) return(ffka(X, tochar)) else
     return(as.character.factor(X))
@@ -511,15 +521,15 @@ as_character_factor <- function(X, keep.attr = TRUE) {
   res
 }
 
-as.numeric_factor <- function(X, keep.attr = TRUE) {
-  .Deprecated(msg = "'as.numeric_factor' was renamed to 'as_numeric_factor'. It will be removed end of 2023, see help('collapse-renamed').")
-  as_numeric_factor(X, keep.attr)
-}
-
-as.character_factor <- function(X, keep.attr = TRUE) {
-  .Deprecated(msg = "'as.character_factor' was renamed to 'as_character_factor'. It will be removed end of 2023, see help('collapse-renamed').")
-  as_character_factor(X, keep.attr)
-}
+# as.numeric_factor <- function(X, keep.attr = TRUE) {
+#   .Deprecated(msg = "'as.numeric_factor' was renamed to 'as_numeric_factor'. It will be removed end of 2023, see help('collapse-renamed').")
+#   as_numeric_factor(X, keep.attr)
+# }
+#
+# as.character_factor <- function(X, keep.attr = TRUE) {
+#   .Deprecated(msg = "'as.character_factor' was renamed to 'as_character_factor'. It will be removed end of 2023, see help('collapse-renamed').")
+#   as_character_factor(X, keep.attr)
+# }
 
 
 setRnDF <- function(df, nm) `attr<-`(df, "row.names", nm)
@@ -560,6 +570,7 @@ ckmatch <- function(x, table, e = "Unknown columns:", ...) if(anyNA(m <- fmatch(
 
 cols2int <- function(cols, x, nam, topos = TRUE) {
  if(is.numeric(cols)) {
+   if(length(cols) == 0L) return(integer(0L))
    l <- length(unclass(x)) # length(nam) ?
    if(cols[1L] < 0L) { # This is sufficient to check negative indices: No R function allows subsetting mixing positive and negative indices.
      if(-bmin(cols) > l) stop("Index out of range abs(1:length(x))")
@@ -622,7 +633,7 @@ cols2intrmgn <- function(gn, cols, x) {
     return(which(cols))
   }
   if(is.null(cols)) return(seq_along(unclass(x))[-gn])
-  if(is.numeric(cols) && cols[1L] < 0) {
+  if(is.numeric(cols) && length(cols) && cols[1L] < 0) {
     res <- logical(length(unclass(x)))
     res[cols] <- TRUE
     res[gn] <- FALSE
@@ -705,6 +716,11 @@ tochar <- function(x) if(is.character(x)) x else as.character(x)  # if(is.object
 #   substr(args, 2, nc) # 3, nc-1 for no brackets !
 # }
 
+switch_msg <- function(msg, which = NULL) {
+  if(is.null(which)) stop(msg)
+  switch(which, error = stop(msg), message = message(msg), warning = warning(msg))
+}
+
 unused_arg_action <- function(call, ...) {
   wo <- switch(getOption("collapse_unused_arg_action"), none = 0L, message = 1L, warning = 2L, error = 3L,
                stop("Unused argument encountered. Please instruct collapse what to do about unused arguments by setting
