@@ -13,7 +13,7 @@ do_collapse_mask <- function(clpns, mask) {
   }
   # This now does the preprocessing (interpreting keywords and changing internal optimization flags as required)
   if(any(mask == "all")) mask <- c("helper", "manip", "special", "fast-fun", if(length(mask) > 1L) mask[mask != "all"] else NULL)
-  manipfun <- c("fsubset", "ftransform", "ftransform<-", "ftransformv", "fcompute", "fcomputev", "fselect", "fselect<-", "fgroup_by", "fgroup_vars", "fungroup", "fsummarise", "fsummarize", "fmutate", "frename", "findex_by", "findex")
+  manipfun <- c("fsubset", "fslice", "fslicev", "ftransform", "ftransform<-", "ftransformv", "fcompute", "fcomputev", "fselect", "fselect<-", "fgroup_by", "fgroup_vars", "fungroup", "fsummarise", "fsummarize", "fmutate", "frename", "findex_by", "findex")
   helperfun <- c("fdroplevels", "finteraction", "fnlevels", "fmatch", "funique", "fnunique", "fduplicated", "fcount", "fcountv", "fquantile", "frange", "fdist", "fnrow", "fncol") # , "fdim": Problem of infinite recursion...
   specialfun <- c("n", "table", "%in%")
   if(any(mask == "helper")) mask <- unique.default(c(helperfun, mask[mask != "helper"]))
@@ -118,7 +118,7 @@ do_collapse_restore_exports <- function(clpns) {
 .onLoad <- function(libname, pkgname) {
 
   res <- .Call(C_collapse_init, "init.success")
-  if(!is.character(res) || res != "init.success") stop("collapse not succesfully loaded!")
+  if(!is.character(res) || res != "init.success") stop("collapse not successfully loaded!")
 
   # https://stackoverflow.com/questions/12598242/global-variables-in-packages-in-r
   # https://stackoverflow.com/questions/49056642/r-how-to-make-variable-available-to-namespace-at-loading-time?noredirect=1&lq=1
