@@ -3,43 +3,29 @@
  and licensed under a Mozilla Public License 2.0 (MPL-2.0) license.
 */
 
-#include <R.h>
-#define USE_RINTERNALS
-#include <Rinternals.h>
-#include <stdint.h> // for uint64_t rather than unsigned long long
+#ifndef DATATABLE_H  // Check if DATATABLE_H is not defined
+#define DATATABLE_H  // Define DATATABLE_H
+
+// #define USE_RINTERNALS
+#include "base_radixsort.h"
+// #include <stdint.h> // for uint64_t rather than unsigned long long
 #include <stdbool.h>
 // #include "types.h"
 
-// data.table depends on R>=3.0.0 when R_xlen_t was introduced
-// Before R 3.0.0, RLEN used to be switched to R_len_t as R_xlen_t wasn't available.
-// We could now replace all RLEN with R_xlen_t directly. Or keep RLEN for the shorter
-// name so as not to have to check closely one letter difference R_xlen_t/R_len_t. We
-// might also undefine R_len_t to ensure not to use it.
-typedef R_xlen_t RLEN;
-
-// #define PRId64 "lld" // needed in rbindlist CHECK_RANGE macro -> disabled right now..
-#define IS_ASCII(x) (LEVELS(x) & 64)
 #define IS_TRUE(x)  (TYPEOF(x)==LGLSXP && LENGTH(x)==1 && LOGICAL(x)[0]==TRUE)
 #define IS_FALSE(x) (TYPEOF(x)==LGLSXP && LENGTH(x)==1 && LOGICAL(x)[0]==FALSE)
 #define IS_TRUE_OR_FALSE(x) (TYPEOF(x)==LGLSXP && LENGTH(x)==1 && LOGICAL(x)[0]!=NA_LOGICAL)
 #define SIZEOF(x) sizes[TYPEOF(x)]
 #define TYPEORDER(x) typeorder[x]
 
+// Needed for match.c and join.c
+#define NEED2UTF8(s) !(IS_ASCII(s) || (s)==NA_STRING || IS_UTF8(s))
+#define ENC2UTF8(s) (!NEED2UTF8(s) ? (s) : mkCharCE(translateCharUTF8(s), CE_UTF8))
+
 // for use with bit64::integer64
 #define NA_INTEGER64  INT64_MIN
 #define MAX_INTEGER64 INT64_MAX
 
-// Backport macros added to R in 2017 so we don't need to update dependency from R 3.0.0
-#ifndef MAYBE_REFERENCED
-# define MAYBE_REFERENCED(x) ( NAMED(x) > 0 )
-#endif
-
-#ifndef ALTREP
-#define ALTREP(x) 0  // for R<3.5.0, see issue #2866 and grep for "ALTREP" to see comments where it's used
-#endif
-
-#define SEXPPTR(x) ((SEXP *)DATAPTR(x))  // to avoid overhead of looped STRING_ELT and VECTOR_ELT
-
 // init.c // https://stackoverflow.com/questions/1410563/what-is-the-difference-between-a-definition-and-a-declaration
 extern SEXP char_integer64;
 extern SEXP char_nanotime;
@@ -48,18 +34,12 @@ extern SEXP char_ordered;
 extern SEXP char_dataframe;
 extern SEXP char_datatable;
 extern SEXP char_sf;
-// not currently needed (base_radixsort uses install), but perhaps later..
 extern SEXP sym_sorted;
-// extern SEXP sym_maxgrpn;
-// extern SEXP sym_starts;
-// extern SEXP char_starts;
 extern SEXP sym_index;
 extern SEXP sym_index_df;
-extern SEXP sym_inherits;
 extern SEXP sym_sf_column;
 extern SEXP SelfRefSymbol;
 extern SEXP sym_datatable_locked;
-// extern SEXP sym_collapse_DT_alloccol;
 
 // data.table_init.c
 SEXP collapse_init(SEXP mess);
@@ -72,6 +52,8 @@ extern size_t sizes[100];  // max appears to be FUNSXP = 99, see Rinternals.h
 extern size_t typeorder[100];
 
 // data.table_utils.c
+int need2utf8(SEXP x);
+SEXP coerceUtf8IfNeeded(SEXP x);
 SEXP setnames(SEXP x, SEXP nam);
 bool allNA(SEXP x, bool errorForBadType);
 SEXP allNAv(SEXP x, SEXP errorForBadType);
@@ -95,3 +77,4 @@ void writeValue(SEXP target, SEXP source, const int from, const int n);
 void savetl_init(void), savetl(SEXP s), savetl_end(void);
 SEXP rbindlist(SEXP l, SEXP usenamesArg, SEXP fillArg, SEXP idcolArg);
 
+#endif // End of DATATABLE_H guard
