@@ -3,10 +3,9 @@
  and licensed under a GPL-3.0 license.
 */
 
+#include "collapse_c.h"
 #include "kit.h"
 
-#define SEXPPTR_RO(x) ((const SEXP *)DATAPTR_RO(x))  // to avoid overhead of looped VECTOR_ELT
-
 // ****************************************
 // This function groups a single vector
 // ****************************************
@@ -27,7 +26,7 @@ SEXP dupVecIndex(SEXP x) {
   } else if(tx == INTSXP) {
     if(isFactor(x) || inherits(x, "qG")) {
       tx = 1000;
-      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, sym_n_groups)) + 2;
       anyNA = !inherits(x, "na.included");
     } else {
       int *restrict p = INTEGER(x);
@@ -54,7 +53,7 @@ SEXP dupVecIndex(SEXP x) {
   } else if (tx == LGLSXP) {
     M = 3;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values, table has size M
   SEXP ans_i = PROTECT(allocVector(INTSXP, n));
   int *restrict pans_i = INTEGER(ans_i), g = 0;
   size_t id = 0;
@@ -175,9 +174,9 @@ SEXP dupVecIndex(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR_RO(x);
     for (int i = 0; i != n; ++i) {
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) {
           pans_i[i] = pans_i[h[id]-1]; // h[id];
@@ -191,9 +190,8 @@ SEXP dupVecIndex(SEXP x) {
     }
   } break;
   }
-  Free(h);
-  SEXP ngroups_sym = install("N.groups");
-  setAttrib(ans_i, ngroups_sym, ScalarInteger(g));
+  R_Free(h);
+  setAttrib(ans_i, sym_n_groups, ScalarInteger(g));
   UNPROTECT(1);
   return ans_i;
 }
@@ -216,7 +214,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
   } else if(tx == INTSXP) {
     if(isFactor(x) || inherits(x, "qG")) {
       tx = 1000;
-      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, sym_n_groups)) + 2;
     } else {
       int *p = INTEGER(x);
       if(n > 10 && (NOGE(p[0], n) || NOGE(p[n/2], n) || NOGE(p[n-1], n))) goto bigint;
@@ -225,7 +223,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
   } else if (tx == LGLSXP) {
     M = 3;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values, table has size M
   SEXP ans_i = PROTECT(allocVector(INTSXP, n));
   int *restrict pans_i = INTEGER(ans_i), g = 0;
   size_t id = 0;
@@ -261,7 +259,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
             goto ibl;
           } // else, we move forward to the next slot, until we find an empty one... We need to keep checking against the values,
           // because if we found the same value before, we would also have put it in another slot after the initial one with the same hash value.
-          if(++id >= nu) id = 0; // ++iid; iid %= nu; // # nocov
+          if(++iid >= nu) iid = 0; // ++iid; iid %= nu; // # nocov
         } // We put the index into the empty slot.
         h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
         pans_i[i] = ++g; // h[id];
@@ -338,13 +336,13 @@ SEXP dupVecIndexKeepNA(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR_RO(x);
     for (int i = 0; i != n; ++i) {
       if(px[i] == NA_STRING) {
         pans_i[i] = NA_INTEGER;
         continue;
       }
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) {
           pans_i[i] = pans_i[h[id]-1]; // h[id];
@@ -358,9 +356,8 @@ SEXP dupVecIndexKeepNA(SEXP x) {
     }
   } break;
   }
-  Free(h);
-  SEXP ngroups_sym = install("N.groups");
-  setAttrib(ans_i, ngroups_sym, ScalarInteger(g));
+  R_Free(h);
+  setAttrib(ans_i, sym_n_groups, ScalarInteger(g));
   UNPROTECT(1);
   return ans_i;
 }
@@ -382,8 +379,8 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
   int both_discr = (tx == LGLSXP || (tx == INTSXP && (isFactor(x) || inherits(x, "qG")))) &&
                    (ty == LGLSXP || (ty == INTSXP && (isFactor(y) || inherits(y, "qG"))));
   if(both_discr) {
-    K = tx == LGLSXP ? 1 : isFactor(x) ? nlevels(x) : asInteger(getAttrib(x, install("N.groups")));
-    K2 = ty == LGLSXP ? 1 : isFactor(y) ? nlevels(y) : asInteger(getAttrib(y, install("N.groups")));
+    K = tx == LGLSXP ? 1 : isFactor(x) ? nlevels(x) : asInteger(getAttrib(x, sym_n_groups));
+    K2 = ty == LGLSXP ? 1 : isFactor(y) ? nlevels(y) : asInteger(getAttrib(y, sym_n_groups));
     if(tx == LGLSXP || !inherits(x, "na.included")) {
       K += 1; anyNA += 1;
     }
@@ -405,7 +402,7 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
     }
   }
 
-  int *restrict h = (int*)Calloc(M, int), g = 0, hid = 0; // Table to save the hash values, table has size M
+  int *restrict h = (int*)R_Calloc(M, int), g = 0, hid = 0; // Table to save the hash values, table has size M
   size_t id = 0;
 
   if(both_discr) {
@@ -452,9 +449,9 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
           }
         } break;
         case STRSXP: {
-          const SEXP *restrict px = STRING_PTR_RO(x), *restrict py = STRING_PTR_RO(y);
+          const SEXP *restrict px = SEXPPTR_RO(x), *restrict py = SEXPPTR_RO(y);
           for (int i = 0; i != n; ++i) {
-            id = HASH(64988430769U * ((intptr_t)px[i] & 0xffffffff) + ((intptr_t)py[i] & 0xffffffff), K); // Best combination it seems
+            id = HASH(64988430769U * ((uintptr_t)px[i] & 0xffffffff) + ((uintptr_t)py[i] & 0xffffffff), K); // Best combination it seems
             while(h[id]) {
               hid = h[id]-1;
               if(px[hid] == px[i] && py[hid] == py[i]) {
@@ -513,12 +510,12 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         }
         // Second case: real and string
       } else if ((tx == REALSXP && ty == STRSXP) || (tx == STRSXP && ty == REALSXP)) {
-        const SEXP *restrict ps = STRING_PTR_RO(tx == STRSXP ? x : y);
+        const SEXP *restrict ps = SEXPPTR_RO(tx == STRSXP ? x : y);
         const double *restrict pr = REAL_RO(tx == REALSXP ? x : y);
         union uno tpv;
         for (int i = 0; i != n; ++i) {
           tpv.d = pr[i];
-          id = HASH((tpv.u[0] + tpv.u[1]) * ((intptr_t)ps[i] & 0xffffffff), K); // Best combination it seems
+          id = HASH((tpv.u[0] + tpv.u[1]) * ((uintptr_t)ps[i] & 0xffffffff), K); // Best combination it seems
           while(h[id]) {
             hid = h[id]-1;
             if(REQUAL(pr[hid], pr[i]) && ps[hid] == ps[i]) { // Seems comparing reals is faster..
@@ -534,9 +531,9 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
         // Third case: integer and string
       } else if((tx == INTSXP && ty == STRSXP) || (tx == STRSXP && ty == INTSXP)) {
         const int *restrict pi = INTEGER_RO(tx == INTSXP ? x : y);
-        const SEXP *restrict ps = STRING_PTR_RO(tx == STRSXP ? x : y);
+        const SEXP *restrict ps = SEXPPTR_RO(tx == STRSXP ? x : y);
         for (int i = 0; i != n; ++i) {
-          id = HASH(pi[i] * ((intptr_t)ps[i] & 0xffffffff), K);
+          id = HASH(pi[i] * ((uintptr_t)ps[i] & 0xffffffff), K);
           while(h[id]) {
             hid = h[id]-1;
             if(pi[hid] == pi[i] && ps[hid] == ps[i]) {
@@ -553,9 +550,8 @@ SEXP dupVecIndexTwoVectors(SEXP x, SEXP y) {
     }
   }
 
-  Free(h);
-  SEXP ngroups_sym = install("N.groups");
-  setAttrib(ans, ngroups_sym, ScalarInteger(g));
+  R_Free(h);
+  setAttrib(ans, sym_n_groups, ScalarInteger(g));
   UNPROTECT(1);
   return ans;
 }
@@ -579,7 +575,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
   size_t M;
   if (tx == INTSXP || tx == STRSXP || tx == REALSXP || tx == CPLXSXP ) {
     if(tx == INTSXP && (isFactor(x) || inherits(x, "qG"))) {
-      K = isFactor(x) ? nlevels(x)+1 : asInteger(getAttrib(x, install("N.groups")))+1;
+      K = isFactor(x) ? nlevels(x)+1 : asInteger(getAttrib(x, sym_n_groups))+1;
       anyNA = !inherits(x, "na.included");
       if((size_t)K * ng <= (size_t)n * 3) {
         tx = 1000;
@@ -599,7 +595,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
   } else if (tx == LGLSXP) {
     M = (size_t)ng * 3 + 1;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *restrict h = (int*)Calloc(M, int), g = 0, hid = 0; // Table to save the hash values, table has size M
+  int *restrict h = (int*)R_Calloc(M, int), g = 0, hid = 0; // Table to save the hash values, table has size M
   size_t id = 0;
   switch (tx) {
   case LGLSXP:
@@ -702,10 +698,10 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
     }
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR_RO(x);
     const unsigned int mult = (M-1) / ng; // -1 because C is zero indexed
     for (int i = 0; i != n; ++i) {
-      id = (pidx[i]*mult) ^ HASH(((intptr_t) px[i] & 0xffffffff), K); // HASH(((intptr_t) px[i] & 0xffffffff) ^ pidx[i], K) + pidx[i];
+      id = (pidx[i]*mult) ^ HASH(((uintptr_t) px[i] & 0xffffffff), K); // HASH(((uintptr_t) px[i] & 0xffffffff) ^ pidx[i], K) + pidx[i];
       while(h[id]) {
         hid = h[id]-1;
         if(pidx[hid] == pidx[i] && px[hid] == px[i]) {
@@ -720,7 +716,7 @@ int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n,
     }
   } break;
   }
-  Free(h);
+  R_Free(h);
   return g;
 }
 
@@ -733,7 +729,7 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
   int l = length(X), islist = TYPEOF(X) == VECSXP,
     start = asLogical(starts), size = asLogical(sizes), nprotect = 0;
   // Better not exceptions to fundamental algorithms, when a couple of user-level functions return qG objects...
-  // if(islist == 0 && OBJECT(X) != 0 && inherits(X, "qG") && inherits(X, "na.included")) return X; // return "qG" objects
+  // if(islist == 0 && isObject(X) && inherits(X, "qG") && inherits(X, "na.included")) return X; // return "qG" objects
   const SEXP *px = islist ? SEXPPTR_RO(X) : &X;
   SEXP idx = islist == 0 ? dupVecIndex(X) : l > 1 ? dupVecIndexTwoVectors(px[0], px[1]) : dupVecIndex(px[0]);
   if(isNull(idx)) { // One of the vectors is complex valued
@@ -741,8 +737,8 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
     l += 1; px -= 1;
   } else if(!(islist && l > 2) && start == 0 && size == 0) return idx; // l == 1 &&
   PROTECT(idx); ++nprotect;
-  SEXP sym_ng = install("N.groups"), res;
-  int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx);
+  SEXP res;
+  int ng = asInteger(getAttrib(idx, sym_n_groups)), n = length(idx);
   if(islist && l > 2) {
     SEXP ans = PROTECT(allocVector(INTSXP, n)); ++nprotect;
     int i = 2, *pidx = INTEGER(idx), *pans = INTEGER(ans);
@@ -755,40 +751,46 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
       }
     }
     res = i % 2 ? ans : idx;
-    setAttrib(res, sym_ng, ScalarInteger(ng));
+    setAttrib(res, sym_n_groups, ScalarInteger(ng));
   } else res = idx;
   // Cumpoting group starts and sizes attributes
   if(start || size) {
     PROTECT(res); ++nprotect;
     int *pres = INTEGER(res);
     if(start && size) { // Protect res ??
-      SEXP gs, st, starts_sym = install("starts"), sizes_sym = install("group.sizes");
-      setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
-      setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
-      int *pgs = INTEGER(gs), *pst = INTEGER(st);
-      memset(pgs, 0, sizeof(int) * ng); --pgs;
-      memset(pst, 0, sizeof(int) * ng); --pst;
-      for(int i = 0; i != n; ++i) {
-        ++pgs[pres[i]];
-        if(pst[pres[i]] == 0) pst[pres[i]] = i + 1;
+      SEXP gs, st;
+      setAttrib(res, sym_starts, st = allocVector(INTSXP, ng));
+      setAttrib(res, sym_group_sizes, gs = allocVector(INTSXP, ng));
+      if(ng > 0) {
+        int *pgs = INTEGER(gs), *pst = INTEGER(st);
+        memset(pgs, 0, sizeof(int) * ng); --pgs;
+        memset(pst, 0, sizeof(int) * ng); --pst;
+        for(int i = 0; i != n; ++i) {
+          ++pgs[pres[i]];
+          if(pst[pres[i]] == 0) pst[pres[i]] = i + 1;
+        }
       }
     } else if(start) {
-      SEXP st, starts_sym = install("starts");
-      setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
-      int *pst = INTEGER(st), k = 0;
-      memset(pst, 0, sizeof(int) * ng); --pst;
-      for(int i = 0; i != n; ++i) {
-        if(pst[pres[i]] == 0) {
-          pst[pres[i]] = i + 1;
-          if(++k == ng) break;
+      SEXP st;
+      setAttrib(res, sym_starts, st = allocVector(INTSXP, ng));
+      if(ng > 0) {
+        int *pst = INTEGER(st), k = 0;
+        memset(pst, 0, sizeof(int) * ng); --pst;
+        for(int i = 0; i != n; ++i) {
+          if(pst[pres[i]] == 0) {
+            pst[pres[i]] = i + 1;
+            if(++k == ng) break;
+          }
         }
       }
     } else {
-      SEXP gs, sizes_sym = install("group.sizes");
-      setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
-      int *pgs = INTEGER(gs);
-      memset(pgs, 0, sizeof(int) * ng); --pgs;
-      for(int i = 0; i != n; ++i) ++pgs[pres[i]];
+      SEXP gs;
+      setAttrib(res, sym_group_sizes, gs = allocVector(INTSXP, ng));
+      if(ng > 0) {
+        int *pgs = INTEGER(gs);
+        memset(pgs, 0, sizeof(int) * ng); --pgs;
+        for(int i = 0; i != n; ++i) ++pgs[pres[i]];
+      }
     }
   }
   UNPROTECT(nprotect);
@@ -804,23 +806,25 @@ SEXP groupAtVec(SEXP X, SEXP starts, SEXP naincl) {
   SEXP idx = nain ? dupVecIndex(X) : dupVecIndexKeepNA(X);
   if(start == 0) return idx;
   PROTECT(idx);
-  SEXP st, sym_ng = install("N.groups"), starts_sym = install("starts");
-  int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx), *pidx = INTEGER(idx);
-  setAttrib(idx, starts_sym, st = allocVector(INTSXP, ng));
-  int *pst = INTEGER(st), k = 0;
-  memset(pst, 0, sizeof(int) * ng); --pst;
-  if(nain) {
-    for(int i = 0; i != n; ++i) {
-      if(pst[pidx[i]] == 0) {
-        pst[pidx[i]] = i + 1;
-        if(++k == ng) break;
+  SEXP st;
+  int ng = asInteger(getAttrib(idx, sym_n_groups)), n = length(idx), *pidx = INTEGER(idx);
+  setAttrib(idx, sym_starts, st = allocVector(INTSXP, ng));
+  if(ng > 0) {
+    int *pst = INTEGER(st), k = 0;
+    memset(pst, 0, sizeof(int) * ng); --pst;
+    if(nain) {
+      for(int i = 0; i != n; ++i) {
+        if(pst[pidx[i]] == 0) {
+          pst[pidx[i]] = i + 1;
+          if(++k == ng) break;
+        }
       }
-    }
-  } else {
-    for(int i = 0; i != n; ++i) {
-      if(pidx[i] != NA_INTEGER && pst[pidx[i]] == 0) {
-        pst[pidx[i]] = i + 1;
-        if(++k == ng) break;
+    } else {
+      for(int i = 0; i != n; ++i) {
+        if(pidx[i] != NA_INTEGER && pst[pidx[i]] == 0) {
+          pst[pidx[i]] = i + 1;
+          if(++k == ng) break;
+        }
       }
     }
   }
@@ -848,7 +852,7 @@ SEXP funiqueC(SEXP x) {
   } else if(tx == INTSXP) {
     if(isFactor(x) || inherits(x, "qG")) {
       tx = 1000;
-      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, sym_n_groups)) + 2;
     } else {
       int *p = INTEGER(x);
       if(n > 10 && (NOGE(p[0], n) || NOGE(p[n/2], n) || NOGE(p[n-1], n))) goto bigint;
@@ -857,7 +861,7 @@ SEXP funiqueC(SEXP x) {
   } else if (tx == LGLSXP) {
     M = 3;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values, table has size M
   int *restrict st = (int*)R_alloc((tx == LGLSXP || tx == 1000) ? (int)M : n, sizeof(int));
   int g = 0, nprotect = 0;
   size_t id = 0;
@@ -884,7 +888,7 @@ SEXP funiqueC(SEXP x) {
         if(++g == ng) break;
       }
     }
-    Free(h);
+    R_Free(h);
     if(g == n) return x;
     PROTECT(res = allocVector(tx == LGLSXP ? LGLSXP : INTSXP, g)); ++nprotect;
     int *restrict pres = INTEGER(res);
@@ -917,7 +921,7 @@ SEXP funiqueC(SEXP x) {
         ibbl:;
       }
     }
-    Free(h);
+    R_Free(h);
     if(g == n) {
       UNPROTECT(nprotect);
       return x;
@@ -940,7 +944,7 @@ SEXP funiqueC(SEXP x) {
       st[g++] = i;
       rbl:;
     }
-    Free(h);
+    R_Free(h);
     if(g == n) {
       UNPROTECT(nprotect);
       return x;
@@ -974,7 +978,7 @@ SEXP funiqueC(SEXP x) {
       st[g++] = i;
       cbl:;
     }
-    Free(h);
+    R_Free(h);
     if(g == n) {
       UNPROTECT(nprotect);
       return x;
@@ -984,9 +988,9 @@ SEXP funiqueC(SEXP x) {
     for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
   } break;
   case STRSXP: {
-    const SEXP *restrict px = STRING_PTR(x);
+    const SEXP *restrict px = SEXPPTR_RO(x);
     for (int i = 0; i != n; ++i) {
-      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      id = HASH(((uintptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
         if(px[h[id]-1] == px[i]) goto sbl;
         if(++id >= M) id = 0; // # nocov
@@ -995,13 +999,13 @@ SEXP funiqueC(SEXP x) {
       st[g++] = i;
       sbl:;
     }
-    Free(h);
+    R_Free(h);
     if(g == n) {
       UNPROTECT(nprotect);
       return x;
     }
     PROTECT(res = allocVector(STRSXP, g)); ++nprotect;
-    SEXP *restrict pres = STRING_PTR(res);
+    SEXP *restrict pres = SEXPPTR(res);
     for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
   } break;
   }
@@ -1037,9 +1041,9 @@ SEXP funiqueC(SEXP x) {
 //     K++;
 //   }
 //   R_xlen_t count = 0;
-//   int *restrict h = (int*) Calloc(M, int);
+//   int *restrict h = (int*) R_Calloc(M, int);
 //   const int *restrict v = INTEGER(mlv);
-//   int *restrict pans = (int*) Calloc(len_i, int);
+//   int *restrict pans = (int*) R_Calloc(len_i, int);
 //   size_t id = 0;
 //
 //       for (R_xlen_t i = 0; i < len_i; ++i) {
@@ -1063,7 +1067,7 @@ SEXP funiqueC(SEXP x) {
 //         count++;
 //         label2:;
 //       }
-//     Free(h);
+//     R_Free(h);
 //     UNPROTECT(1);
 //     SEXP indx = PROTECT(allocVector(INTSXP, count));
 //     int ct = 0;
@@ -1074,7 +1078,7 @@ SEXP funiqueC(SEXP x) {
 //       }
 //     }
 //     SEXP output = PROTECT(subSetRowDataFrame(x, indx));
-//     Free(pans);
+//     R_Free(pans);
 //     UNPROTECT(2);
 //     return output;
 // }
@@ -1114,7 +1118,7 @@ SEXP funiqueC(SEXP x) {
 //     K++;
 //   }
 //   R_xlen_t count = 0;
-//   int *restrict h = (int*) Calloc(M, int);
+//   int *restrict h = (int*) R_Calloc(M, int);
 //   const int *restrict v = INTEGER(mlv);
 //   size_t id = 0;
 //   for (R_xlen_t i = 0; i < len_i; ++i) {
@@ -1137,7 +1141,7 @@ SEXP funiqueC(SEXP x) {
 //     count++;
 //     label2:;
 //   }
-//   Free(h);
+//   R_Free(h);
 //   UNPROTECT(1);
 //   return ScalarInteger(count);
 // }
